/*
    PicProbe for Sharp Linux Zaurus
    Copyright (C) 2005  NODA Atsushi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <qtextcodec.h>
#include <qlayout.h>
#include <qtimer.h>
#include <qpainter.h>

#ifdef QWS
#include <qpe/qpeapplication.h>
#include <qpe/qpemessagebox.h>
#include <qpe/qcopenvelope_qws.h>
#include <qpe/config.h>
#include <qpe/global.h>
#else
#include <stdlib.h> // for alice
#include <qapplication.h>
#include <qmessagebox.h>
#endif

#include <qmessagebox.h>
#include <math.h>
#include "main.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <stdio.h>
#include <unistd.h>

const int timeOut = 50; // msec
const int timeNoData = 500; // msec

ImageWindows::ImageWindows(QWidget *parent, const char *name, WFlags f)
  :QWidget(parent,name,f)
{
  setBackgroundMode(NoBackground);
  _buffer.resize(width(),height());
  _buffer.fill(white);
}

void ImageWindows::paintEvent( QPaintEvent* event)
{
  bitBlt(this, 0, 0, &_buffer);
}

void ImageWindows::resizeEvent( QResizeEvent* event)
{
  QPixmap save( _buffer);
  _buffer.resize(event->size());
  _buffer.fill(white);
  bitBlt(this, 0, 0, &save);
}

void ImageWindows::flush()
{
  bitBlt(this, 0, 0, &_buffer);
}

void ImageWindows::dspCenter(const QString &t)
{
  int fontSize = 72;
#ifdef QWS
  QFont    f( "lcfont", fontSize);
#else
  QFont    f;
  f.setPointSize( fontSize);
#endif
  QFontMetrics fm(f);
  int fth = fm.height();
  int ftw = fm.width(t);
  int w0 = width();
  int h0 = height();
  _buffer.fill(white);
  QPainter bufferpainter;
  bufferpainter.begin(&_buffer);
  bufferpainter.setFont( f );
  //  bufferpainter.setPen(gray);
  bufferpainter.setPen(red);
  bufferpainter.drawText((w0 - ftw) / 2, (h0 + fth) / 2,t);
  bufferpainter.end();
  bitBlt(this, 0, 0, &_buffer);
}

bool ImageWindows::dspData(bool flgD,const unsigned int * adata,const bool * flgAD)
{
  int fontSize;
  int fontSizeS;
  fontSize = 48;
  fontSizeS = 24;
  int w0 = width();
  int h0 = height();
  _buffer.fill(white);
  QPainter bufferpainter;
  bufferpainter.begin(&_buffer);
  bufferpainter.setPen(black);
  bufferpainter.drawLine(0,0,w0 - 1,0);
  bufferpainter.drawLine(0,0,0,h0);
  bufferpainter.drawLine(w0 - 1,0,w0 - 1,h0);
  bufferpainter.drawLine(0,h0 - 1,w0 - 1,h0 - 1);
  bufferpainter.drawLine(0,h0 / 2,w0 - 1,h0 / 2);
  bufferpainter.drawLine(w0 / 3,0,w0 / 3,h0);
  bufferpainter.drawLine(w0 * 2 / 3,0,w0 * 2/ 3,h0);
  for (int i = 0;i < 5; i++) {
    int x0 = w0 / 3 * (i % 3);
    int y0 = h0 / 2 * (i / 3);
    char buf[16];
    int j = i;
    if (i >= 4) j = i - 1;
#ifdef QWS
    QFont f( "lcfont", fontSize);
    QFont fs( "lcfont", fontSizeS);
#else
    QFont    f;
    f.setPointSize( fontSize);
    QFont    fs;
    fs.setPointSize( fontSizeS);
#endif
    QFontMetrics fm(f);
    int fth = fm.height();
    QFontMetrics fms(fs);
    int fths = fms.height();
    bufferpainter.setFont( f );
    bufferpainter.setPen(black);
    if (flgAD[i]) {
      sprintf(buf,"AN%d %dpin",j,7 - i);
      bufferpainter.drawText(x0 + 10, y0 + fth + 10,buf);
      int mx0 = x0 + 20;
      int mx1 = x0 + w0 / 3 - 20;
      int my0 = y0 + fth + 20;
      int my1 = y0 + h0 / 2 - fths - 20;
      double r = (my1 - my0) * 0.85;
      bufferpainter.drawLine(mx0,my0,mx0,my1);
      bufferpainter.drawLine(mx0,my1,mx1,my1);
      bufferpainter.drawLine(mx0,my0,mx1,my0);
      bufferpainter.drawLine(mx1,my0,mx1,my1);
      bufferpainter.setPen(red);
      int mmx = (mx0 + mx1) / 2 + r * sin(1.8 * (adata[i] / 1023.0 - 0.5));
      int mmy = my1 - r * cos(1.8 * (adata[i] / 1023.0 - 0.5));
      bufferpainter.drawLine((mx0 + mx1) / 2 ,my1,mmx,mmy);
      sprintf(buf,"         %4d",adata[i]);
      bufferpainter.setPen(black);
      bufferpainter.setFont( fs );
      bufferpainter.drawText(x0, y0 + h0 / 2 - 10,buf);
    } else if (flgD) {
      sprintf(buf,"GP%d %dpin",i,7 - i);
      bufferpainter.drawText(x0 + 10, y0 + fth + 10,buf);
      bufferpainter.setPen(black);
      if(adata[i]) {
        bufferpainter.setBrush(green);
        sprintf(buf,"H");
      } else {
        bufferpainter.setBrush(red);
        sprintf(buf,"L");
      }
      int r = w0 / 3 / 3;
      bufferpainter.drawEllipse(x0 + w0 / 6 - r / 2, y0 + fth + 30,r,r);
      int ftw = fm.width(buf);
      bufferpainter.drawText(x0 + w0 / 6 - ftw / 2, y0 + h0 / 2 - 10,buf);
    } else {
      sprintf(buf," %dpin",7 - i);
      bufferpainter.drawText(x0, y0 + fth,buf);
      sprintf(buf," No Data");
      bufferpainter.drawText(x0, y0 + 2 * fth,buf);
    }
  }
  bufferpainter.end();
  bitBlt(this, 0, 0, &_buffer);

  return true;
}

void ImageWindows::clearImage()
{
  _buffer.fill(white);
  bitBlt(this, 0, 0, &_buffer);
}

MainForm *mainForm;

MainForm::MainForm()
{
  QTextCodec * codec = QTextCodec::codecForName( "eucJP" );
  setCaption("Pic Probe");
  QVBoxLayout *qvbl = new QVBoxLayout(this);
  imageWindows = new ImageWindows( this, "" );
  qvbl->addWidget(imageWindows);
#ifdef QWS
  fd = open("/dev/ttyS1",O_RDONLY|O_NONBLOCK|O_NOCTTY);
#else
  fd = open("/dev/ttyS1",O_RDONLY|O_NONBLOCK|O_NOCTTY);
#endif
  if(fd != -1) {
    struct termios options;
    tcgetattr(fd, &options);

    options.c_iflag = IGNBRK;
    options.c_cflag = (CS8|CREAD|CLOCAL|CRTSCTS);
    options.c_oflag = (NL0|CR0|TAB0|BS0|VT0|FF0);
    options.c_lflag = 0;

    cfsetispeed(&options, B9600);
    cfsetospeed(&options, B9600);

    options.c_cc[VTIME] = 0;
    options.c_cc[VMIN] = 1;
    tcflush(fd, TCIFLUSH);
    tcsetattr(fd, TCSANOW, &options);
  } else {
    printf("Can not open IO\n");
  }
  internalTimer = new QTimer( this );
  internalTimer->start( timeOut );
  connect(internalTimer, SIGNAL(timeout()),this, SLOT(timeout()));
}

MainForm::~MainForm() {
#ifdef QWS
#endif
}

void MainForm::dspDataClear()
{
  pDataBuf = 0;
}

void MainForm::dspDataSet(const unsigned char *b,int n)
{
  pDataBuf = n;
  for(int i = 0; i < n;i++)
    dataBuf[i] = b[i];
}

bool MainForm::dspData()
{
  if (pDataBuf == 0) return false;
  bool flgAD[5];
  unsigned int adata[5];
  bool flgD = false;
  unsigned char ddata = 0;
  int i;
  for (i = 0; i < 5;i++) {
    flgAD[i] = false;
    adata[i] = 0;
  }
  i = 0;
  while (i < pDataBuf) {
    if ((dataBuf[i] & 0xe0) == 0xc0) {
      if ((i >= (pDataBuf - 1)) || (dataBuf[i + 1] & 0x80))
        return false;
      int j = (dataBuf[i] & 0x18) >> 3;
      if (j == 3)
        j = 4;
      flgAD[j] = true;
      adata[j] = ((dataBuf[i] & 0x07) << 7) | dataBuf[i + 1];
      i++;
    } else if ((dataBuf[i] & 0xe0) == 0xa0) {
      flgD = true;
      ddata = dataBuf[i] & 0x1f;
    } else {
      return false;
    }
    i++;
  }
  int j = 0;
  for(i = 0; i < 5; i++) {
    if (! flgAD[i]) {
      if ((ddata >> i) & 1) {
        adata[i] = 1;
      } else {
        j++;
      }
    }
  }
  if ((j == 0) && (! flgD)) return false;
  return imageWindows->dspData(flgD,adata,flgAD);
}

bool MainForm::getData()
{
  static unsigned char dataBuf[noOfBuf];
  static unsigned int pDataBuf = 0;
  unsigned char inbuf[256];
  dspDataClear();
  if (fd != -1) {
    int res = read(fd,inbuf,255);
    if (res > 0) {
      for (int j = 0;j < res;j++) {
        if (pDataBuf == noOfBuf) {
          for(int i = 0;i < (noOfBuf - 1);i++)
            dataBuf[i] = dataBuf[i + 1];
          pDataBuf = noOfBuf - 1;
        }
        if (inbuf[j] == 0xff) {
          dspDataSet(dataBuf,pDataBuf);
          pDataBuf = 0;
        } else {
          dataBuf[pDataBuf] = inbuf[j];
          pDataBuf ++;
        }
      }
      return dspData();
    }
  }
  return false;
}

void MainForm::timeout()
{
  static int waitData = 0;
  static bool flgNodata = false;
  if (getData()) {
    waitData = 0;
    flgNodata = false;
  } else {
    if (waitData > (timeNoData / timeOut)) {
      if (! flgNodata)
        imageWindows->dspCenter("No Data");
      flgNodata = true;
    } else {
      waitData++;
    }
  }
}

int main(int argc, char** argv)
{
#ifdef QWS
  QPEApplication a( argc, argv);
  mainForm = new MainForm();
  a.showMainDocumentWidget(mainForm);
  int r = a.exec();
  return r;
#else
  QApplication a( argc, argv);
  mainForm = new MainForm();
  mainForm->setGeometry( 10, 10, 780, 550);
  a.setMainWidget(mainForm);
  mainForm->show();
  int r = a.exec();
  return r;
#endif
}
